// Fortran rule:
// IMPLICIT: default IMPLICIT REAL(A-H, O-Z), INTEGER(I-N)
// here, we wil have IMPLICIT DOUBLE PRECISION(a-h,o-z), INTEGER(i-n)

#include <stack-c.h>
#include <sciprint.h>
#include <Scierror.h>
#include <MALLOC.h>

#include <stdlib.h>
#include <string.h>

//#define DEBUG 1
#define DBGPRINTF sciprint
//#define DBGPRINTF printf

#include "helper.h"

#ifdef WIN32
#define my_import __declspec(dllimport)
#define my_export __declspec(dllexport)
#else
#define my_import extern
#define my_export extern
#endif

// Order of the parameters:
#define X0_IN      1
#define FOBJ_IN    2
#define GOBJ_IN    3
#define NCON_IN    4
#define VUB_IN     5
#define VLB_IN     6
#define ITMAX_IN   7
#define PARAM_IN   8
#define LAST_PARAM 8

#define X_OPT_OUT  9
#define F_OPT_OUT  10
#define DF_OPT_OUT 11
#define G_OPT_OUT  12
#define DG_OPT_OUT 13
#define IC_OPT_OUT 14

extern void C2F(get_ct)(double *);
extern void C2F(get_obj)(double *);
extern void C2F(get_igoto)(int *);
extern void C2F(get_nac)(int *);
extern void C2F(set_delfun)(double *);
extern void C2F(set_dabfun)(double *);
extern void C2F(set_fdch)(double *);
extern void C2F(set_fdchm)(double *);
extern void C2F(set_ct)(double *);
extern void C2F(set_ctmin)(double *);
extern void C2F(set_ctl)(double *);
extern void C2F(set_ctlmin)(double *);
extern void C2F(set_alphax)(double *);
extern void C2F(set_abobj1)(double *);
extern void C2F(set_theta)(double *);
extern void C2F(set_obj)(double *);
extern void C2F(set_ndv)(int *);
extern void C2F(set_ncon)(int *);
extern void C2F(set_nside)(int *);
extern void C2F(set_iprint)(int *);
extern void C2F(set_nfdg)(int *);
extern void C2F(set_nscal)(int *);
extern void C2F(set_linobj)(int *);
extern void C2F(set_itmax)(int *);
extern void C2F(set_itrm)(int *);
extern void C2F(set_icndir)(int *);
extern void C2F(set_igoto)(int *);
extern void C2F(set_nac)(int *);
extern void C2F(set_info)(int *);
extern void C2F(set_infog)(int *);
extern void C2F(set_iter)(int *);

typedef struct {
  double delfun;
  double dabfun;
  double fdch;
  double fdchm;
  double ct;
  double ctmin;
  double ctl;
  double ctlmin;
  double alphax;
  double abobj1;
  double theta;
  double obj;
  int    ndv;
  int    ncon;
  int    nside;
  int    iprint;
  int    nfdg;
  int    nscal;
  int    linobj;
  int    itmax;
  int    itrm;
  int    icndir;
  int    igoto;
  int    nac;
  int    info;
  int    infog;
  int    iter;
} MY_CNMN1;

my_export MY_CNMN1 C2F(cnmn1);

typedef struct {
  double dm1;
  double dm2;
  double dm3;
  double dm4;
  double dm5;
  double dm6;
  double dm7;
  double dm8;
  double dm9;
  double dm10;
  double dm11;
  double dm12;
  double dct;
  double dctl;
  double phi;
  double abobj;
  double cta;
  double ctam;
  double ctbm;
  double obj1;
  double slope;
  double dx;
  double dx1;
  double fi;
  double xi;
  double dftdf1;
  double alp;
  double fff;
  double a1;
  double a2;
  double a3;
  double a4;
  double f1;
  double f2;
  double f3;
  double f4;
  double cv1;
  double cv2;
  double cv3;
  double cv4;
  double app;
  double alpca;
  double alpfes;
  double alpln;
  double alpmin;
  double alpnc;
  double alpsav;
  double alpsid;
  double alptot;
  double rspace;
  int idm1;
  int idm2;
  int idm3;
  int jdir;
  int iobj;
  int kobj;
  int kcount;
  int ncal[2];
  int nfeas;
  int mscal;
  int ncobj;
  int nvc;
  int kount;
  int icount;
  int igood1;
  int igood2;
  int igood3;
  int igood4;
  int ibest;
  int iii;
  int nlnc;
  int jgoto;
  int ispace[2];
} MY_CONSAV;

my_export MY_CONSAV C2F(consav);

//                       X         VLB       VUB       G         SCAL      DF        A         S         G1        G2        B          C
extern  void C2F(conmin)(double *, double *, double *, double *, double *, double *, double *, double *, double *, double *, double *, double *,
                         int *, int *, int *, int *, int *, int *, int *, int *);
//                       ISC    IC     MS1    N1     N2     N3     N4     N5

// Example of settings from: example1.f       example2.f       example3.f
// INFOG  = 0                                 Idem             Idem
// INFO   = 0                                 Idem             Idem
// NFDG   = 0                                 NFDG   = 1       NFDG  = 2
// IPRINT = 2                                 IPRINT = 1       Idem
// NDV    = 4                                 Idem             NDV   = 2
// ITMAX  = 40                                Idem             Idem
// NCON   = 3                                 Idem             NCON  = 6
// NSIDE  = 0                                 Idem             NSIDE = 1
// ICNDIR = 0                                 Idem             Idem
// NSCAL  = 0                                 Idem             Idem
// FDCH   = 0.0                               Idem             Idem
// FDCHM  = 0.0                               Idem             Idem
// CT     = 0.0                               Idem             Idem
// CTMIN  = 0.0                               Idem             Idem
// CTL    = 0.0                               Idem             Idem
// CTLMIN = 0.0                               Idem             Idem
// THETA  = 0.0                               Idem             Idem
// DELFUN = 0.0                               Idem             Idem
// DABFUN = 0.0                               Idem             Idem
// LINOBJ = 0.0                               Idem             LINOBJ = 1
// ITRM   = 0                                 Idem             Idem
// N1     = 6                                 Idem             N1     = 4
// N2     = 11                                Idem             N2     = 10
// N3     = 11                                Idem             N3     = 10
// N4     = 11                                Idem             N4     = 10
// N5     = 22                                Idem             N5     = 20
// ALPHAX = 0.0                               Idem             Idem
// ABOBJ1 = 0.0                               Idem             Idem
// CTL    = 0.0                               Idem             Idem
// DO 5 I=1,NDV
//   X(I)   = 1.0                             Idem             Idem
//   VLB(I) = -99999.                         Idem             VLB(I) = 0.001
//   VUB(I) =  99999.                         Idem             VUB(I) = 1.0E+10
// 5 CONTINUE
//
// DO 6 J=1,NCON
//   ISC(J) = 0                               Idem             Idem
// 6 CONTINUE

// Prototypes for objective function and constraints:
// Objective function: function [f,df] = fobj(x)
// x  = [nx][1]
// f  = [1][1]
// df = [nx][1]
// Constraints:        function [g,dg,ic] = constr(x,ct)
// x  = [nx][1]
// ct = [1][1]
// g  = [nac][1]
// dg = [nx][nac] 
// ic = [nac][1]  - contains the index of active constraints
// The constraints tested must be
// g_i(x)<=ct

int sci_conmin(char * fname)
{
  int n_x,      m_x,      l_x;      // Input - X + ndv
  int fobj_lhs, fobj_rhs, l_fobj;   // Input - f_obj
  int gobj_lhs, gobj_rhs, l_gobj;   // Input - g_obj + ncon
  int n_vub,    m_vub,    l_vub;    // Input - VUB + NSIDE = 1 if vub and / or vlb are present
  int n_vlb,    m_vlb,    l_vlb;    // Input - VLB
  int n_itmax,  m_itmax,  l_itmax;  // Input - ITMAX
  int n_ncon,   m_ncon,   l_ncon;   // Input - NCON

  // Temporary variables for the call to objective function and constraints
  int n_x1_tmp    = 0, m_x1_tmp    = 0, l_x1_tmp    = 0;
  int n_x2_tmp    = 0, m_x2_tmp    = 0, l_x2_tmp    = 0;
  int n_x3_tmp    = 0, m_x3_tmp    = 0, l_x3_tmp    = 0;
  int n_fobj_tmp  = 0, m_fobj_tmp  = 0, l_fobj_tmp  = 0;
  int n_dfobj_tmp = 0, m_dfobj_tmp = 0, l_dfobj_tmp = 0;
  int n_gobj_tmp  = 0, m_gobj_tmp  = 0, l_gobj_tmp  = 0;
  int n_dgobj_tmp = 0, m_dgobj_tmp = 0, l_dgobj_tmp = 0;
  int n_ic        = 0, m_ic        = 0, l_ic        = 0;
  // Output variables
  int n_x_opt,    m_x_opt,    l_x_opt;  // Output
  int n_f_opt,    m_f_opt,    l_f_opt;  // Output - optional
  int n_df_opt,   m_df_opt,   l_df_opt; // Output - optional
  int n_g_opt,    m_g_opt,    l_g_opt;  // Output - optional
  int n_dg_opt,   m_dg_opt,   l_dg_opt; // Output - optional
  int n_ic_res,   m_ic_res,   l_ic_res; // Output - optional

  int n1,n2,n3,n4,n5;
  int ibegin; // To be used in SciFunction
  int i,j;
  int tmp_int, tmp_int_2, tmp_res;
  double tmp_dbl;

  int    * pIsc  = NULL;
  double * pScal = NULL;
  int szIsc = 0, szScal = 0;
  int Isc_mallocated = 0, Scal_mallocated = 0;

  double * G   = NULL;
  double * A   = NULL;
  double * S   = NULL;
  double * G1  = NULL;
  double * G2  = NULL;
  double * B   = NULL;
  double * C   = NULL;
  double * DF  = NULL;
  int    * MS1 = NULL;

  int     m_param, n_param, l_param;
  int     m_label, n_label, pos_label;
  int     m_tmp,   n_tmp,   l_tmp;
  char ** LabelList = NULL;

  // Order of the parameters:
  // x0
  // fobj
  // gobj
  // ncon
  // nacmx
  // vub
  // vlb
  // itmax
  // isc
  // scal
  // nscal
  // nfdg
  // icndir
  // fdch
  // fdchm
  // ct
  // ctmin
  // ctl
  // ctlmin
  // theta
  // delfun
  // dabfun
  // linobj
  // itrm
  // alphax
  // abobj1
  // infog
  // info
  // iprint

#ifdef DEBUG
  DBGPRINTF("conmin_optim: Lhs = %d Rhs = %d\n",Lhs,Rhs);
#endif

  if (Rhs<LAST_PARAM)
    {
      Scierror(999,"conmin_optim: %d parameters are required\n",LAST_PARAM);
      return 0;
    } /* End If */

  ////////////////////////
  // Get the parameters //
  ////////////////////////

  GetRhsVar(X0_IN,   "d", &n_x,      &m_x,      &l_x);
  tmp_int = n_x*m_x;
  C2F(set_ndv)(&tmp_int);
  //C2F(cnmn1).ndv   = n_x*m_x;

  GetRhsVar(FOBJ_IN, "f", &fobj_lhs, &fobj_rhs, &l_fobj);
  GetRhsVar(GOBJ_IN, "f", &gobj_lhs, &gobj_rhs, &l_gobj);

  /////////////////////////////////////////////
  // Initialization of some "size" variables //
  /////////////////////////////////////////////

  GetRhsVar(NCON_IN, "i", &n_ncon, &m_ncon, &l_ncon);
  C2F(set_ncon)(istk(l_ncon));
  //C2F(cnmn1).ncon = *istk(l_ncon);

  GetRhsVar(VUB_IN, "d", &n_vub, &m_vub, &l_vub);
  if (n_vub*m_vub!=n_x*m_x)
    {
      Scierror(999,"conmin_optim: upper must have the same length as x\n");
      return 0;
    }

  GetRhsVar(VLB_IN, "d", &n_vlb, &m_vlb, &l_vlb);
  if (n_vlb*m_vlb!=n_x*m_x)
    {
      Scierror(999,"conmin_optim: upper must have the same length as x\n");
      return 0;
    }

  GetRhsVar(ITMAX_IN, "i", &n_itmax, &m_itmax, &l_itmax);
  C2F(set_itmax)(istk(l_itmax));
  //C2F(cnmn1).itmax = *istk(l_itmax);

  // YC: si nvub != nvlb mettre le plus petit  +inf ou -inf
  // YC: si nvub != nvlb != ndv mettre tout  +inf et -inf et taille de ndv
  // C2F(cnmn1).nside = n_vub * m_vub;
  tmp_int = 2 * n_x * m_x;
  C2F(set_nside)(&tmp_int);
  //C2F(cnmn1).nside = 2 * C2F(cnmn1).ndv;
  //if (C2F(cnmn1).nside < (n_vlb * m_vlb)) C2F(cnmn1).nside = n_vlb * m_vlb;

  ///////////////////////////////////////
  // Get the parameters from the plist //
  ///////////////////////////////////////

  GetRhsVar(PARAM_IN, MATRIX_ORIENTED_TYPED_LIST_DATATYPE, &m_param, &n_param, &l_param);
  GetListRhsVar(PARAM_IN, 1, MATRIX_OF_STRING_DATATYPE, &m_label, &n_label, &LabelList);

  //
  // First, we process the solver option to allocate the good solver
  //

  if (strcmp(LabelList[0],"plist") != 0) 
    {
      Scierror(999,"Argument %d is not a plist\r\n", PARAM_IN);
      return 0;
    }

  tmp_int_2 = *istk(l_ncon) + 2 * n_x * m_x + 1;
  //tmp_int_2 = C2F(cnmn1).ncon + C2F(cnmn1).nside + 1;
  GET_PARAM_INT("nacmx", tmp_int, tmp_int_2, tmp_res);
  n3 = tmp_int;

  GET_PARAM_VECT_INT("isc",pIsc,szIsc,tmp_res);

  if (!((tmp_res!=-1) && (szIsc==n_x*m_x)))
    {
      // We must allocate our own pIsc
      pIsc = (int *)MALLOC(n_x*m_x*sizeof(int));
      for(i=0;i<n_x*m_x;i++) pIsc[i] = 0;
      Isc_mallocated = 1;
    }

  GET_PARAM_VECT_DOUBLE("scal",pScal,szScal,tmp_res);

  if (!((tmp_res!=-1) && (szScal==n_x*m_x)))
    {
      // We must allocate our own pScal
      pScal = (double *)MALLOC(n_x*m_x*sizeof(double));
      for(i=0;i<n_x*m_x;i++) pScal[i] = 1.0;
      Scal_mallocated = 1;
    }

  GET_PARAM_INT("nscal",tmp_int,0,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).nscal = tmp_int;
  if (tmp_res!=-1) C2F(set_nscal)(&tmp_int);

  GET_PARAM_INT("nfdg",tmp_int,0,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).nfdg = tmp_int;
  if (tmp_res!=-1) C2F(set_nfdg)(&tmp_int);

  GET_PARAM_INT("icndir",tmp_int,n_x*m_x+1,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).icndir = tmp_int;
  if (tmp_res!=-1) C2F(set_icndir)(&tmp_int);

  GET_PARAM_DOUBLE("fdch",tmp_dbl,0.01,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).fdch = tmp_dbl;
  if (tmp_res!=-1) C2F(set_fdch)(&tmp_dbl);

  GET_PARAM_DOUBLE("fdchm",tmp_dbl,0.01,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).fdchm = tmp_dbl;
  if (tmp_res!=-1) C2F(set_fdchm)(&tmp_dbl);

  GET_PARAM_DOUBLE("ct",tmp_dbl,-0.1,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).ct = tmp_dbl;
  if (tmp_res!=-1) C2F(set_ct)(&tmp_dbl);

  GET_PARAM_DOUBLE("ctmin",tmp_dbl,0.004,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).ctmin = tmp_dbl;
  if (tmp_res!=-1) C2F(set_ctmin)(&tmp_dbl);

  GET_PARAM_DOUBLE("ctl",tmp_dbl,-0.01,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).ctl = tmp_dbl;
  if (tmp_res!=-1) C2F(set_ctl)(&tmp_dbl);

  GET_PARAM_DOUBLE("ctlmin",tmp_dbl,0.001,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).ctlmin = tmp_dbl;
  if (tmp_res!=-1) C2F(set_ctlmin)(&tmp_dbl);

  GET_PARAM_DOUBLE("theta",tmp_dbl,1.0,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).theta = tmp_dbl;
  if (tmp_res!=-1) C2F(set_theta)(&tmp_dbl);

  GET_PARAM_DOUBLE("delfun",tmp_dbl,0.001,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).delfun = tmp_dbl;
  if (tmp_res!=-1) C2F(set_delfun)(&tmp_dbl);

  GET_PARAM_DOUBLE("dabfun",tmp_dbl,0.001,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).dabfun = tmp_dbl;
  if (tmp_res!=-1) C2F(set_dabfun)(&tmp_dbl);

  GET_PARAM_INT("linobj",tmp_int,0,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).linobj = tmp_int;
  if (tmp_res!=-1) C2F(set_linobj)(&tmp_int);

  GET_PARAM_INT("itrm",tmp_int,3,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).itrm = tmp_int;
  if (tmp_res!=-1) C2F(set_itrm)(&tmp_int);

  GET_PARAM_DOUBLE("alphax",tmp_dbl,0.3,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).alphax = tmp_dbl;
  if (tmp_res!=-1) C2F(set_alphax)(&tmp_dbl);

  GET_PARAM_DOUBLE("abobj1",tmp_dbl,0.2,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).abobj1 = tmp_dbl;
  if (tmp_res!=-1) C2F(set_abobj1)(&tmp_dbl);

  GET_PARAM_INT("infog",tmp_int,0,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).infog = tmp_int;
  if (tmp_res!=-1) C2F(set_infog)(&tmp_int);

  GET_PARAM_INT("info",tmp_int,1,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).info = tmp_int;
  if (tmp_res!=-1) C2F(set_info)(&tmp_int);

  GET_PARAM_INT("iprint",tmp_int,0,tmp_res);
  //if (tmp_res!=-1) C2F(cnmn1).iprint = tmp_int;
  if (tmp_res!=-1) C2F(set_iprint)(&tmp_int);

  // Verification: 
  // size(x) = size(vlb) = size(vub) 
  // size(x) = size(scal) ou (size(scal)==0 et nscal = 0)
  // size(isc) = ncon ou 0

  n1 = n_x*m_x + 2;
  n2 = *istk(l_ncon) + 2*n_x*m_x;
  n4 = (n3>n_x*m_x)?n3:n_x*m_x;
  n5 = 2*n4;

#ifdef DEBUG
  DBGPRINTF("x_start = [ ");
  for(j=0;j<n_x*m_x;j++) DBGPRINTF("%f ",*stk(l_x+j));
  DBGPRINTF("]\n");
  
  DBGPRINTF("vub = [ ");
  for(j=0;j<n_x*m_x;j++) DBGPRINTF("%f ",*stk(l_vub+j));
  DBGPRINTF("]\n");
  
  DBGPRINTF("vlb = [ ");
  for(j=0;j<n_x*m_x;j++) DBGPRINTF("%f ",*stk(l_vlb+j));
  DBGPRINTF("]\n");
  
  DBGPRINTF("isc[0]  = %d\n", *pIsc);
  DBGPRINTF("scal[0] = %f\n", *pScal);
  DBGPRINTF("nscal   = %d\n", *istk(l_nscal));
  DBGPRINTF("infog   = %d\n", *istk(l_infog));
  DBGPRINTF("info    = %d\n", *istk(l_info));
  DBGPRINTF("nfdg    = %d\n", *istk(l_nfdg));
  DBGPRINTF("iprint  = %d\n", *istk(l_iprint));
  DBGPRINTF("itmax   = %d\n", *istk(l_itmax));
  DBGPRINTF("icndir  = %d\n", *istk(l_icndir));
  DBGPRINTF("fdch    = %f\n", *stk(l_fdch));
  DBGPRINTF("fdchm   = %f\n", *stk(l_fdchm));
  DBGPRINTF("ct      = %f\n", *stk(l_ct));
  DBGPRINTF("ctmin   = %f\n", *stk(l_ctmin));
  DBGPRINTF("ctl     = %f\n", *stk(l_ctl));
  DBGPRINTF("ctlmin  = %f\n", *stk(l_ctlmin));
  DBGPRINTF("theta   = %f\n", *stk(l_theta));
  DBGPRINTF("delfun  = %f\n", *stk(l_delfun));
  DBGPRINTF("dabfun  = %f\n", *stk(l_dabfun));
  DBGPRINTF("linobj  = %d\n", *istk(l_linobj));
  DBGPRINTF("itrm    = %d\n", *istk(l_itrm));
  DBGPRINTF("alphax  = %f\n", *stk(l_alphax));
  DBGPRINTF("abobj1  = %f\n", *stk(l_abobj1));
  DBGPRINTF("nacmx   = %d\n", *istk(l_nacmx));
  DBGPRINTF("ncon    = %d\n", *istk(l_ncon));
  DBGPRINTF("ndv     = %d\n", n_x*m_x);
#endif

  /////////////////////////////////////////////////////
  // First call to objective function and constraint //
  /////////////////////////////////////////////////////

  ////////////////////////////////
  // Call to objective function //
  ////////////////////////////////

  ibegin = Rhs + 1;

  // Create the first variable: x
  n_x1_tmp = n_x; m_x1_tmp = m_x;
  CreateVar(ibegin,"d",&n_x1_tmp,&m_x1_tmp,&l_x1_tmp);
  memcpy(stk(l_x1_tmp),stk(l_x),n_x*m_x*sizeof(double));

  // Create a fake 2nd variable on the stack. This will allow to call GetRhsVar to retrieve the 2nd output argument
  n_x2_tmp = 1; m_x2_tmp = 1;
  CreateVar(ibegin+1,"d",&n_x2_tmp,&m_x2_tmp,&l_x2_tmp);

  // Call to the scilab function
  SciFunction(&ibegin,&l_fobj,&fobj_lhs,&fobj_rhs);
  if (Err>0) 
    {
      sciprint("conmin: error when calling objective function\n");
      return 0;
    } /* End If */
  
  // YC: we should add a check on fobj_lhs and fobj_rhs to check the scilab prototype of fobj

  // Get fobj
  GetRhsVar(ibegin,  "d",&n_fobj_tmp, &m_fobj_tmp, &l_fobj_tmp);
  // Get gradient of fobj
  GetRhsVar(ibegin+1,"d",&n_dfobj_tmp,&m_dfobj_tmp,&l_dfobj_tmp);
  
  // Fill C2F(cnmn1).obj: objective function value
  C2F(set_obj)(stk(l_fobj_tmp));
  //C2F(cnmn1).obj = *stk(l_fobj_tmp);
  
  // DF(N1) Analytic gradient of the objective function for the current decision variables, X(I).  DF(I) contains the partial derivative
  //        of OBJ with respect to X(I).  Calculate DF(I), I = 1, NDV if INFO = 3 or INFO = 4 and if NFDG = 0 or NFDG = 2.
  
  DF = (double *)malloc(n1*sizeof(double));
  
  // Fill DF(N1): gradient of objective function value
  memcpy(DF,stk(l_dfobj_tmp),n_x*m_x*sizeof(double));

  ////////////////////////
  // Call to constraint //
  ////////////////////////

  ibegin = Rhs + 1; // YC: for SciFunction

  // YC: test nombre de param en entree (2) / sortie (3)

  // First parameter: the vector x
  n_x1_tmp = n_x; m_x1_tmp = m_x;
  CreateVar(ibegin,"d",&n_x1_tmp,&m_x1_tmp,&l_x1_tmp);
  memcpy(stk(l_x1_tmp),stk(l_x),n_x*m_x*sizeof(double));

  // Second parameter: the scalar CT (Constraint Threshold)
  n_x2_tmp = 1; m_x2_tmp = 1;
  CreateVar(ibegin+1,"d",&n_x2_tmp,&m_x2_tmp,&l_x2_tmp);
  //*stk(l_x2_tmp) = C2F(cnmn1).ct;
  C2F(get_ct)(stk(l_x2_tmp));
  
  // Third parameter: a fake parameter which will receive IC (index of active constraints)
  n_x3_tmp = 1; m_x3_tmp = 1;
  CreateVar(ibegin+2,"d",&n_x3_tmp,&m_x3_tmp,&l_x3_tmp);

  SciFunction(&ibegin,&l_gobj,&gobj_lhs,&gobj_rhs);
  if (Err>0) 
    {
      sciprint("conmin: error when calling constraints function\n");
      return 0;
    } /* End If */

  // YC: we should add a check on gobj_lhs and gobj_rhs to check the scilab prototype of gobj

  // After the first call to the constraints, we get the number of constraints via the first variable returned by constraint function
  // Get gobj
  GetRhsVar(ibegin,  "d",&n_gobj_tmp, &m_gobj_tmp, &l_gobj_tmp);
  // Get gradient of gobj
  GetRhsVar(ibegin+1,"d",&n_dgobj_tmp,&m_dgobj_tmp,&l_dgobj_tmp);
  // Get indexes of active constraints
  GetRhsVar(ibegin+2,"i",&n_ic,       &m_ic,       &l_ic);
  // dgobj_tmp is of size [ndv][nac]
  //C2F(cnmn1).nac = n_ic*m_ic;
  tmp_int = n_ic*m_ic;
  C2F(set_nac)(&tmp_int);
  
  if (tmp_int>0)
    {
      // In scilab, the index start from 1. We need an index which start from 0
      for(j=0;j<tmp_int;j++)
	{
	  *istk(l_ic+j) = *istk(l_ic+j) - 1;
	} /* End For */
    } /* End If */

  ///////////////////////
  // Memory allocation //
  ///////////////////////

  // S(N1)    Move direction in the NDV-dimensional optimization space. S(I) gives the rate at which variable X(I) changes with respect toALPHA.  
  S  = (double *)malloc(n1*sizeof(double));
  
  // G1(N2)   Not used if NCON = NSIDE = NSCAL = 0.  Used for temporary storage of constraint values G(J), J = 1, NCON and decision
  //          variables X(I), I = 1, NDV.
  G1 = (double *)malloc(n2*sizeof(double));
  
  // G2(N2)   Not used if NCON = NSIDE = 0.  Used for temporary storage of constraint values G(J), J = 1, NCON.
  G2 = (double *)malloc(n2*sizeof(double));
  
  // B(N3,N3) Not used if NCON = NSIDE = 0.  Used in determining direction vector S for constrained minimization problems.  Array B may
  //          be used for temporary storage in external routine SUB1.
  B = (double *)malloc(n3*n3*sizeof(double));

  // C(N4)    Not used in NCON = NSIDE = 0.  Used with array B in determining direction vector S for constrained minimization problems.  Used
  //          for temporary storage of vector X if NSCAL.NE.0. routine SUB1.
  C   = (double *)malloc(n4*sizeof(double));

  // MS1(N5)  Not used if NCON = NSIDE = 0.  Used with array B in determining direction vector S for constrained minimization problems.  Array
  //          MS1 may be used for temporary storage in external routine SUB1.
  MS1 = (int *)malloc(n5*sizeof(int));

  // G(N2)  Not used if NCON = NSIDE = 0.  Vector containing all constraint functions, G(J), J = 1, NCON for current decision variables, X.
  //        Calculate G(J), J = 1, NCON if INFO = 2.
  G  = (double *)malloc(n2*sizeof(double));
  // Fill G(N2): value of constraint
  for(i=0;i<*istk(l_ncon);i++) G[i] = *stk(l_gobj_tmp+i);

  // A(N1,N3)  Not used if NCON = NSIDE = 0.  Gradients of active or violated constraints, for current decision variables, X(I).
  //           A(J,I) contains the gradient of the Jth active or violated constraint, G(J), with respect to the Ith decision variable,
  //           X(I) for J = 1, NAC and I = 1, NDV.  Calculate if INFO = 4 and NFDG = 0.
  // YC: adress if 0 bytes after a block of size 48 alloc'd
  A = (double *)malloc(n1*n3*sizeof(double));

  // Fill A(N1,N3): matrix of gradient of active constraint
  if (tmp_int>0) // tmp_int is still equal to C2F(cnmn1).nac
    {
      memcpy(A,stk(l_dgobj_tmp),n_dgobj_tmp*m_dgobj_tmp*sizeof(double));
    } /* End If */

  //C2F(cnmn1).obj   = *stk(l_fobj_tmp);
  //C2F(cnmn1).igoto = 0;
  //C2F(cnmn1).iter  = 0;

  tmp_int = 0;
  C2F(set_obj)(stk(l_fobj_tmp));
  C2F(set_igoto)(&tmp_int);
  C2F(set_iter)(&tmp_int);

  // Start the optimization

  // Optimization loop
  for(i=0;i<*istk(l_itmax);i++)
    {
#ifdef DEBUG
      DBGPRINTF("Iteration %d / %d\n",i,*istk(l_itmax));
#endif
      // Call to objective function and constraint function. 
      // Create a new variable: x1_tmp on top of the stack - outside of the loop
      // Call SciFunction on fobj with ibegin=29
      // Get fobj and dfobj on stack at position 30 and 31
      // Call SciFunction on gobj with ibegin=29
      // Get gobj and dgobj on stack at position 30 and 31 

      if (i>0) 
	{
	  // We skip the first iteration because we already have made a call to objective function and constraints

	  ////////////////////////////////
	  // Call to objective function //
	  ////////////////////////////////

	  ibegin = Rhs + 1; // YC: for SciFunction

	  // First parameter: the vector x
	  n_x1_tmp = n_x; m_x1_tmp = m_x;
	  CreateVar(ibegin,"d",&n_x1_tmp,&m_x1_tmp,&l_x1_tmp);
	  memcpy(stk(l_x1_tmp),stk(l_x),n_x*m_x*sizeof(double));

	  // Create a fake 2nd variable on the stack. This will allow to call GetRhsVar to retrieve the 2nd output argument
	  n_x2_tmp = 1; m_x2_tmp = 1;
	  CreateVar(ibegin+1,"d",&n_x2_tmp,&m_x2_tmp,&l_x2_tmp);

	  SciFunction(&ibegin,&l_fobj,&fobj_lhs,&fobj_rhs);
	  if (Err>0) 
	    {
	      sciprint("conmin: error when calling objective function at iteration %d / %d\n",i,*istk(l_itmax));
	      return 0;
	    } /* End If */

	  // Get fobj
	  GetRhsVar(ibegin,  "d",&n_fobj_tmp, &m_fobj_tmp, &l_fobj_tmp);
	  // Get gradient of fobj
	  GetRhsVar(ibegin+1,"d",&n_dfobj_tmp,&m_dfobj_tmp,&l_dfobj_tmp);

	  // Fill C2F(cnmn1).obj: objective function value
	  //C2F(cnmn1).obj = *stk(l_fobj_tmp);
	  C2F(set_obj)(stk(l_fobj_tmp));
	  // Fill DF(N1): gradient of objective function value
	  memcpy(DF,stk(l_dfobj_tmp),n_x*m_x*sizeof(double));

	  //////////////////////////////////
	  // Call to constraints function //
	  //////////////////////////////////

	  ibegin = Rhs + 1; // YC: for SciFunction

	  // First parameter: the vector x
	  n_x1_tmp = n_x; m_x1_tmp = m_x;
	  CreateVar(ibegin,"d",&n_x1_tmp,&m_x1_tmp,&l_x1_tmp);
	  memcpy(stk(l_x1_tmp),stk(l_x),n_x*m_x*sizeof(double));
	
	  // Second parameter: the scalar CT (Constraint Threshold)
	  n_x2_tmp = 1; m_x2_tmp = 1;
	  CreateVar(ibegin+1,"d",&n_x2_tmp,&m_x2_tmp,&l_x2_tmp);
	  //*stk(l_x2_tmp) = C2F(cnmn1).ct;
	  C2F(get_ct)(stk(l_x2_tmp));
	  
	  // Third parameter: a fake parameter which will receive the IC (index of active constraint) list
	  n_x3_tmp = 1; m_x3_tmp = 1;
	  CreateVar(ibegin+2,"d",&n_x3_tmp,&m_x3_tmp,&l_x3_tmp);

	  SciFunction(&ibegin,&l_gobj,&gobj_lhs,&gobj_rhs);
	  if (Err>0) 
	    {
	      sciprint("conmin: error when calling constraints function at iteration %d / %d\n",i,*istk(l_itmax));
	      return 0;
	    } /* End If */

	  // Now, we get the constraint values and the derivate of the constraints
	  // Get gobj
	  GetRhsVar(ibegin,  "d",&n_gobj_tmp, &m_gobj_tmp, &l_gobj_tmp);
	  // Get gradient of gobj
	  GetRhsVar(ibegin+1,"d",&n_dgobj_tmp,&m_dgobj_tmp,&l_dgobj_tmp);
	  // Get indexes of active constraints
	  GetRhsVar(ibegin+2,"i",&n_ic,       &m_ic,       &l_ic);

	  // dgobj_tmp is of size [ndv][nac]
	  //C2F(cnmn1).nac = n_ic*m_ic;
	  tmp_int = n_ic*m_ic;
	  C2F(set_nac)(&tmp_int);

	  // Fill G(N2): value of constraints
	  memcpy(G,stk(l_gobj_tmp),*istk(l_ncon)*sizeof(double));

	  if (tmp_int>0) // tmp_int is still equal to nac
	    {
	      // In scilab, the index start from 1. We need an index which start from 0
	      for(j=0;j<tmp_int;j++)
		{
		  *istk(l_ic+j) = *istk(l_ic+j) - 1;
		} /* End For */

	      // Fill A(N1,N3): matrix of gradient of active constraint
	      memcpy(A,stk(l_dgobj_tmp),n_dgobj_tmp*m_dgobj_tmp*sizeof(double));
	    } /* End If */
	} /* End If */
      
      //                        X         VLB       VUB       G         SCAL      DF        A         S         G1        G2        B         C
      //extern void C2F(conmin)(double *, double *, double *, double *, double *, double *, double *, double *, double *, double *, double *, double *,
      //                        int *, int *, int *, int *, int *, int *, int *, int *);
      //                        ISC    IC     MS1    N1     N2     N3     N4     N5

      C2F(conmin)(stk(l_x),stk(l_vlb),stk(l_vub),G,pScal,DF,A,S,G1,G2,B,C,pIsc,istk(l_ic),MS1,&n1,&n2,&n3,&n4,&n5);

      //if (C2F(cnmn1).igoto == 0) break; // Optimization has converged
      C2F(get_igoto)(&tmp_int);
      if (tmp_int == 0) break; // Optimization has converged
    } /* End For */

  //////////////////////////
  // return the variables //
  //////////////////////////

  // x_opt
  n_x_opt = n_x; m_x_opt = m_x;
  CreateVar(X_OPT_OUT, "d", &n_x_opt, &m_x_opt, &l_x_opt);
  memcpy(stk(l_x_opt),stk(l_x),n_x*m_x*sizeof(double));

  // f_opt
  n_f_opt = 1; m_f_opt = 1;
  CreateVar(F_OPT_OUT,"d", &n_f_opt, &m_f_opt, &l_f_opt);
  //*stk(l_f_opt) = C2F(cnmn1).obj;
  C2F(get_obj)(stk(l_f_opt));

  // df_opt
  n_df_opt = n_x; m_df_opt = m_x;
  CreateVar(DF_OPT_OUT,"d", &n_df_opt, &m_df_opt, &l_df_opt);
  memcpy(stk(l_df_opt),DF,n_x*m_x*sizeof(double));
  
  // g_opt
  n_g_opt = *istk(l_ncon); m_g_opt = 1;
  CreateVar(G_OPT_OUT, "d", &n_g_opt, &m_g_opt, &l_g_opt);
  memcpy(stk(l_g_opt),G,*istk(l_ncon)*sizeof(double));
  
  // dg_opt
  n_dg_opt = n_dgobj_tmp; m_dg_opt = m_dgobj_tmp;
  CreateVar(DG_OPT_OUT,"d", &n_dg_opt, &m_dg_opt, &l_dg_opt);
  // Now, we stored the gradient of active constraint in the matrix
  memcpy(stk(l_dg_opt),A,n_dgobj_tmp*m_dgobj_tmp*sizeof(double));

  // IC
  C2F(get_nac)(&tmp_int);
  if (tmp_int!=0)
    {
      n_ic_res = tmp_int; m_ic_res = 1;
      CreateVar(IC_OPT_OUT, "i", &n_ic_res, &m_ic_res, &l_ic_res);
      // Now, we stored the gradient of active constraint in the matrix
      memcpy(istk(l_ic_res),istk(l_ic),tmp_int*sizeof(int));
    }
  else
    {
      n_ic_res = 1; m_ic_res = 1;
      CreateVar(IC_OPT_OUT, "i", &n_ic_res, &m_ic_res, &l_ic_res);
      *istk(l_ic_res) = -1;
    }

  LhsVar(1) = X_OPT_OUT;
  LhsVar(2) = F_OPT_OUT;
  LhsVar(3) = DF_OPT_OUT;
  LhsVar(4) = G_OPT_OUT;
  LhsVar(5) = DG_OPT_OUT;
  LhsVar(6) = IC_OPT_OUT;

  ///////////////////////////
  // Free allocated memory //
  ///////////////////////////

  if (G)   free(G);
  if (A)   free(A);
  if (DF)  free(DF);
  if (S)   free(S);
  if (G1)  free(G1);
  if (G2)  free(G2);
  if (B)   free(B);
  if (C)   free(C);
  if (MS1) free(MS1);

  if (Isc_mallocated)  free(pIsc);
  if (Scal_mallocated) free(pScal);

  return 0;
}
